<?php

namespace App\Http\Controllers\Exam;

use App\Http\Controllers\CollegeBaseController;
use Illuminate\Http\Request;
use App\Models\Exam\ExamCreate;
use App\Models\Exam\ExamMode;
use App\Models\Exam\ExamType;
use App\Models\Exam\ExamTerm;
use App\Models\Exam\ExamPaper;
use App\Models\Faculty;
use Carbon\Carbon;
use Session,DB;

class ExamCreateController extends CollegeBaseController
{
    protected $base_route = 'exam.create';
    protected $view_path = 'exam.create';
    protected $panel = 'Exams';
    protected $filter_query = [];

    public function __construct()
    {

    }

    public function index(Request $request)
    {
       
        $dropdowns = $this->get_table_and_dropdown();
        return view(parent::loadDataToView($this->view_path.'.index'), compact('dropdowns'));
    }
    public function store_exam(Request $request)
    {
        $msg=[
            'title.required'=>"Please Enter Title",
            'mode_id.required'=>"Please Select Exam Mode",
            'term_id.required'=>"Please Select Term",
            'type_id.required'=>"Please Select Exam Type",
            'paper_type.required'=>"Please Select Paper Type",
            'faculty_id.required'=>"Please Select ".env('course_label'),
            'section_id.required'=>"Please Select Section",
            'subject_id.required'=>"Please Select Subject",
        ];
        $rules=[
            'title'=>'required',
            'mode_id'=>'required',
            'term_id'=>'required',
            'type_id'=>'required',
            'paper_type'=>'required',
            'faculty_id'=>'required',
            'section_id'=>'required',
            'subject_id'=>'required',
            'title'=>'required',
        ];
        
        $this->validate($request,$rules,$msg);
        $request->request->add(['created_at'=>Carbon::now()]);
        $request->request->add(['created_by'=>auth()->user()->id]);
        $request->request->add(['record_status'=>1]);
        $request->request->add(['branch_id'=>Session::get('activeBranch')]);
        $request->request->add(['session_id'=>Session::get('activeSession')]);
        ExamCreate::create($request->all()); 
         $request->session()->flash($this->message_success, $this->panel.' Added Successfully.');
        return back();
    }
    public function edit_exam(Request $request,$id){
        
        $data['row']=ExamCreate::find($id);

        if(!$data['row']){
            parent::invalidRequest();
        }
        if($request->all() ){
            $msg=[
            'title.required'=>"Please Enter Title",
            ];
            $rules=[
                'title'=>'required',
            ];
            $this->validate($request,$rules,$msg);
            $request->request->add(['updated_at'=>Carbon::now()]);
            $request->request->add(['updated_by'=>auth()->user()->id]);
            $data['row']->update($request->all());
            return redirect()->route($this->base_route)->with('message_success', $this->panel.' Updated Successfully');
        }
        $dropdowns = $this->get_table_and_dropdown($data['row']->term_id,$data['row']->faculty_id,$data['row']->section_id);
        return view(parent::loadDataToView($this->view_path.'.index'),compact('data','id','dropdowns'));
    }          
    public function delete_exam(Request $request, $id)
    {
       $data['row']=ExamCreate::find($id);
        if(!$data['row']){
            parent::invalidRequest();
        }
        $request->request->add(['record_status'=>0]);
            $data['row']->update($request->all());
        return redirect()->route($this->base_route)->with('message_success', $this->panel.' Deleted Successfully');    
    }
    public function change_status(Request $request, $id,$status)
    {
       $data['row']=ExamCreate::find($id);
        if(!$data['row']){
            parent::invalidRequest();
        }
        if((!$data['row']->date || !$data['row']->start_time || !$data['row']->end_time)&& ($status ==1)){
             return redirect()->route($this->base_route.'.edit',['id'=>$id])->with('message_warning','Please Add Date, Start Time, End Time First');
           
        } $request->request->add(['publish_status'=>$status]);
            $data['row']->update($request->all());  
        return redirect()->route($this->base_route)->with('message_success', $this->panel.' Status Updated Successfully');    
    } 
    public function result_status(Request $request, $id,$status)
    {
       $data['row']=ExamCreate::find($id);
        if(!$data['row']){
            parent::invalidRequest();
        }
        $current_date = Carbon::now();
        $exam_end_date = new Carbon($data['row']->date.' '.$data['row']->end_time);

        if(($current_date < $exam_end_date)&& ($status ==1)){
            return redirect()->back()->with('message_warning','Exam not completed yet, Cannot publish result now.');
           
        } $request->request->add(['result_status'=>$status]);
            $data['row']->update($request->all());  
        return redirect()->route($this->base_route)->with('message_success', $this->panel.' Result Status Updated Successfully');    
    }
    public function get_table_and_dropdown($term_id="",$faculty_id="",$section_id=""){
        $data['mode'] = ExamMode::where([
            ['record_status','=',1],
        ])->select('id','title')->pluck('title','id')->toArray();
        $data['mode'] = array_prepend($data['mode'],'--Select Exam Mode--','');

        $data['term'] = ExamTerm::where([
            ['record_status','=',1],
            ['branch_id','=',Session::get('activeBranch')],
            ['session_id','=',Session::get('activeSession')],
        ])->select('id','title')->pluck('title','id')->toArray();
        $data['term'] = array_prepend($data['term'],'--Select Term--','');


        $data['type'] = ExamType::where([
            ['record_status','=',1],
            ['branch_id','=',Session::get('activeBranch')],
            ['session_id','=',Session::get('activeSession')],
        ])
        ->where(function($q)use($term_id){
            if($term_id){
                $q->where('term_id',$term_id);
            }
        })
        ->select('id','title')->pluck('title','id')->toArray();
        $data['type'] = array_prepend($data['type'],'--Select Type--','');


        $data['section'] = DB::table('faculty_semester')
        ->where(function($q)use($faculty_id){
            if($faculty_id){
                $q->where('faculty_id',$faculty_id);
            }
        })
        ->select('sem.id','semester')
        ->leftjoin('semesters as sem','sem.id','=','faculty_semester.semester_id')
        ->pluck('semester','id')->toArray();
        $data['section'] = array_prepend($data['section'],'--Select Section--','');


        $data['subject'] = DB::table('timetable_subjects')
        ->where([
            ['status','=',1],
            ['branch_id','=',Session::get('activeBranch')],
            ['session_id','=',Session::get('activeSession')],
        ])
        ->where(function($q)use($faculty_id,$section_id){
            if($faculty_id){
                $q->where('course_id',$faculty_id);
                $q->where('section_id',$section_id);
            }
        })
        ->select('id','title')
        ->pluck('title','id')->toArray();
        $data['subject'] = array_prepend($data['subject'],'--Select Subject--','');


        
        $data['paper-type'] = ExamPaper::where([
            ['record_status','=',1],
            ['branch_id','=',Session::get('activeBranch')],
            ['session_id','=',Session::get('activeSession')],
        ])->select('id','title')->pluck('title','id')->toArray();
        $data['paper-type'] = array_prepend($data['paper-type'],'--Select Paper Type--','');

        $data['faculty'] = Faculty::where([
            ['status','=',1],
            ['branch_id','=',Session::get('activeBranch')],
        ])->select('id','faculty as title')->orderBy('title','asc')->pluck('title','id')->toArray();
        $data['faculty'] = array_prepend($data['faculty'],'--Select '.env('course_label').'--','');
        
        $data['exam'] = ExamCreate::select('exam_create.*','exam_create.title as exam_title','exam_create.description as exam_description','term.title as term','type.title as type','fac.faculty as faculty','sem.semester as section','mode.title as mode','paper.title as paper','sub.title as subject','max_mark','pass_mark','publish_status')
        ->where([
            ['exam_create.record_status','=',1],
            ['exam_create.branch_id','=',Session::get('activeBranch')],
            ['exam_create.session_id','=',Session::get('activeSession')],
        ])
         ->leftjoin('exam_terms as term','term.id','=','exam_create.term_id')
         ->leftjoin('exam_type as type','type.id','=','exam_create.type_id')
         ->leftjoin('faculties as fac','fac.id','=','exam_create.faculty_id')
         ->leftjoin('semesters as sem','sem.id','=','exam_create.section_id')
         ->leftjoin('exam_modes as mode','mode.id','=','exam_create.mode_id')
         ->leftjoin('exam_papers as paper','paper.id','=','exam_create.paper_type')
         ->leftjoin('timetable_subjects as sub','sub.id','=','exam_create.subject_id')
         ->get();
        return $data;
    }
    public function getExamType(Request $request){
        $data = [];
        $data['error'] = true;
        if($request->term_id){ 
            $data['type'] = ExamType::select('*')->where([
                ['record_status','=',1],
                ['term_id','=',$request->term_id],
            ])->get();
            if(count($data['type'])>0){
                $data['error'] = false;
                $data['msg'] = 'Exam Type Found';
                $data['type'] = $data['type'];
            }else{
                $data['msg'] = 'No Exam Type Found';
            }
        }else{
            $data['msg'] = 'Invalid Request!';
        }
        return response()->JSON(json_encode($data));
    }
    public function getSubject(Request $request){
        $data = [];
        $data['error'] = true;
        if($request->section && $request->faculty && $request->session && $request->branch){ 
            $data['subject'] = DB::table('timetable_subjects')->select('*')->where([
                ['status','=',1],
                ['course_id','=',$request->faculty],
                ['section_id','=',$request->section],
                ['session_id','=',$request->session],
                ['branch_id','=',$request->branch],
            ])->get();
            if(count($data['subject'])>0){
                $data['error'] = false;
                $data['msg'] = 'Subjects Found';
                $data['type'] = $data['subject'];
            }else{
                $data['msg'] = 'No Subject Found';
            }
        }else{
            $data['msg'] = 'Invalid Request!';
        }
        return response()->JSON(json_encode($data));
    }
}